﻿// <copyright file="FacebookImagePrep.cs" company="$registerdorganization$">
// Copyright (c) 2012 . All Right Reserved
// </copyright>
// <author>Matthias</author>
// <email></email>
// <date>2012-07-23</date>
// <summary>A value converter for WPF and Silverlight data binding</summary>

namespace Facebook_Tutorial_2.ImageHelpers
{
    using System;
    using System.Windows.Data;
    using System.Windows.Media.Imaging;
    using System.Windows.Media;
    using System.Windows.Controls;
    using System.IO;
    using ExifLib;

    /// <summary>
    /// A Value converter
    /// </summary>
    public class FacebookImagePrep 
    {
        public static byte[] RotatePhotoResult(Stream chosenPhoto, string originalFilename, double maxSize, int additionalRotation)
        {
            JpegInfo info = ExifReader.ReadJpeg(chosenPhoto, originalFilename);

            int width = info.Width;
            int height = info.Height;
            ExifOrientation orientation = info.Orientation;
            int angle = 0;
            Stream capturedImage;

            switch (info.Orientation)
            {
                case ExifOrientation.TopLeft:
                case ExifOrientation.Undefined:
                    angle = 0;
                    break;
                case ExifOrientation.TopRight:
                    angle = 90;
                    break;
                case ExifOrientation.BottomRight:
                    angle = 180;
                    break;
                case ExifOrientation.BottomLeft:
                    angle = 270;
                    break;
            }


            if (angle > 0d)
                capturedImage = RotateStream(chosenPhoto, angle);
            else
                capturedImage = chosenPhoto;

            BitmapImage bmp = new BitmapImage();
            bmp.SetSource(capturedImage);
            double resizeRatio = 1;
            int resizeHeight = bmp.PixelHeight;
            int resizeWidth = bmp.PixelWidth;

            if (Convert.ToDouble(bmp.PixelHeight) > maxSize || Convert.ToDouble(bmp.PixelWidth) > maxSize)
            {
                if (bmp.PixelHeight > bmp.PixelWidth)
                {
                    resizeRatio = maxSize / Convert.ToDouble(bmp.PixelHeight);
                    resizeHeight = Convert.ToInt16(resizeRatio * Convert.ToDouble(bmp.PixelHeight));
                    resizeWidth = Convert.ToInt16(resizeRatio * Convert.ToDouble(bmp.PixelWidth));
                }
                else
                {
                    resizeRatio = maxSize / Convert.ToDouble(bmp.PixelWidth);
                    resizeHeight = Convert.ToInt16(resizeRatio * Convert.ToDouble(bmp.PixelHeight));
                    resizeWidth = Convert.ToInt16(resizeRatio * Convert.ToDouble(bmp.PixelWidth));
                }
            }

            WriteableBitmap writeBmp = new WriteableBitmap(bmp);
            using (MemoryStream ms = new MemoryStream())
            {
                Extensions.SaveJpeg(writeBmp, ms, resizeWidth, resizeHeight, 0, 100);
                return ms.ToArray();
            }
        }

        private static Stream RotateStream(Stream stream, int angle)
        {
            stream.Position = 0;
            if (angle % 90 != 0 || angle < 0) throw new ArgumentException();
            if (angle % 360 == 0) return stream;

            BitmapImage bitmap = new BitmapImage();
            bitmap.SetSource(stream);
            WriteableBitmap wbSource = new WriteableBitmap(bitmap);

            WriteableBitmap wbTarget = null;
            if (angle % 180 == 0)
            {
                wbTarget = new WriteableBitmap(wbSource.PixelWidth, wbSource.PixelHeight);
            }
            else
            {
                wbTarget = new WriteableBitmap(wbSource.PixelHeight, wbSource.PixelWidth);
            }

            for (int x = 0; x < wbSource.PixelWidth; x++)
            {
                for (int y = 0; y < wbSource.PixelHeight; y++)
                {
                    switch (angle % 360)
                    {
                        case 90:
                            wbTarget.Pixels[(wbSource.PixelHeight - y - 1) + x * wbTarget.PixelWidth] = wbSource.Pixels[x + y * wbSource.PixelWidth];
                            break;
                        case 180:
                            wbTarget.Pixels[(wbSource.PixelWidth - x - 1) + (wbSource.PixelHeight - y - 1) * wbSource.PixelWidth] = wbSource.Pixels[x + y * wbSource.PixelWidth];
                            break;
                        case 270:
                            wbTarget.Pixels[y + (wbSource.PixelWidth - x - 1) * wbTarget.PixelWidth] = wbSource.Pixels[x + y * wbSource.PixelWidth];
                            break;
                    }
                }
            }
            MemoryStream targetStream = new MemoryStream();
            wbTarget.SaveJpeg(targetStream, wbTarget.PixelWidth, wbTarget.PixelHeight, 0, 100);
            return targetStream;
        }

        public static Stream RotateStream(Stream stream, string originalFileName, int rotation)
        {
            JpegInfo info = ExifReader.ReadJpeg(stream, originalFileName);

            int width = info.Width;
            int height = info.Height;
            ExifOrientation orientation = info.Orientation;
            int angle = 0;
            Stream capturedImage;

            switch (info.Orientation)
            {
                case ExifOrientation.TopLeft:
                case ExifOrientation.Undefined:
                    angle = 0;
                    break;
                case ExifOrientation.TopRight:
                    angle = 90;
                    break;
                case ExifOrientation.BottomRight:
                    angle = 180;
                    break;
                case ExifOrientation.BottomLeft:
                    angle = 270;
                    break;
            }

            angle = angle + rotation;
            if (angle >= 360)
                angle = angle - 360;

            if (angle > 0d)
                capturedImage = RotateStream(stream, angle);
            else
                capturedImage = stream;

            return capturedImage;
        }
    }
}
