﻿using Microsoft.Phone.Controls;
using Facebook;
using System.Windows;
using System.Collections.Generic;
using System.Text;
using System;
using Facebook_Tutorial_2.ViewModel;
using Microsoft.Phone.Tasks;

namespace Facebook_Tutorial_2
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        private int _tries = 0;
        private PhotoChooserTask _pct;

        private MainViewModel _viewModel { get { return (MainViewModel)DataContext; } }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {   
            string[] extendedPermissions = new[] { "user_about_me", "publish_stream" };

            var oauth = new FacebookOAuthClient { AppId = App.FacebookAppID, AppSecret = App.FacebookAppSecret };
            var parameters = new Dictionary<string, object>
                            {
                                {"response_type", "token"},
                                {"display", "touch"}                                        
                            };
            if (extendedPermissions != null && extendedPermissions.Length > 0)
            {
                var scope = new StringBuilder();
                scope.Append(string.Join(",", extendedPermissions));
                parameters["scope"] = scope.ToString();
            }

            var loginUrl = oauth.GetLoginUrl(parameters);
            webBrowser.Navigated += new EventHandler<System.Windows.Navigation.NavigationEventArgs>(CheckForAuth);
            webBrowser.Navigate(loginUrl);
        }

        private void CheckForAuth(object sender, System.Windows.Navigation.NavigationEventArgs e)
        {
            FacebookOAuthResult result;
            _tries++;
            
            if (FacebookOAuthResult.TryParse(e.Uri, out result))
            {
                if (result.IsSuccess)
                    _viewModel.SaveAccessToken.Execute(result.AccessToken);
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    HideFacebookLogin.Begin();
                });
            }
            else
            {
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    _viewModel.IsLoading = false;
                });
            }
        }

        private void postImageToWall(object sender, RoutedEventArgs e)
        {
            _viewModel.CommitFacebookPhoto.Execute(null);
        }

        private void postMessageToWall(object sender, RoutedEventArgs e)
        {
            _viewModel.SendFacebookMessage.Execute(null);
        }

        private void TakePicture(object sender, RoutedEventArgs e)
        {
            _pct = new PhotoChooserTask();
            _pct.ShowCamera = true;
            _pct.Completed += new EventHandler<PhotoResult>(_pct_Completed);
            _pct.Show();
        }

        void _pct_Completed(object sender, PhotoResult e)
        {
            _viewModel.ProcessImageResult.Execute(e);
        }

        private void rotateImage(object sender, RoutedEventArgs e)
        {
            _viewModel.RotateCounterClockWise.Execute(null);
        }
    }
}
