﻿using GalaSoft.MvvmLight;
using Facebook_Tutorial_2.Model;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;
using System.IO.IsolatedStorage;
using Facebook;
using System.Windows;
using System.Collections.Generic;
using System.IO;
using Microsoft.Phone.Tasks;
using System.Windows.Media.Imaging;
using System;

namespace Facebook_Tutorial_2.ViewModel
{
    /// <summary>
    /// This class contains properties that the main View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel()
        {
            if (IsInDesignMode)
            {
                // Code runs in Blend --> create design time data.
            }
            else
            {
                // Code runs "for real"
            }
        }

        private FacebookClient _asyncFbClient;
        private string _facebookAccessToken;
        private PhotoResult _rawPhotoResult;
        private byte[] _fbImageHolder;
        private int _additionalRotation = 0;

        #region FacebookMessage (string)
        /// <summary>
        /// The <see cref="FacebookMessage" /> property's name.
        /// </summary>
        public const string FacebookMessagePropertyName = "FacebookMessage";

        private string _facebookMessage = "";

        /// <summary>
        /// Gets the FacebookMessage property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string FacebookMessage
        {
            get
            {
                return _facebookMessage;
            }

            set
            {
                if (_facebookMessage == value)
                {
                    return;
                }

                var oldValue = _facebookMessage;
                _facebookMessage = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(FacebookMessagePropertyName);
            }
        }
        #endregion

        #region FBUser (FacebookUser)
        /// <summary>
        /// The <see cref="FBUser" /> property's name.
        /// </summary>
        public const string FBUserPropertyName = "FBUser";

        private FacebookUser _fbUser = null;

        /// <summary>
        /// Gets the FBUser property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public FacebookUser FBUser
        {
            get
            {
                return _fbUser;
            }

            set
            {
                if (_fbUser == value)
                {
                    return;
                }

                var oldValue = _fbUser;
                _fbUser = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(FBUserPropertyName);
            }
        }
        #endregion

        #region IsLoading (bool)
        /// <summary>
        /// The <see cref="IsLoading" /> property's name.
        /// </summary>
        public const string IsLoadingPropertyName = "IsLoading";

        private bool _isLoading = true;

        /// <summary>
        /// Gets the IsLoading property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsLoading
        {
            get
            {
                return _isLoading;
            }

            set
            {
                if (_isLoading == value)
                {
                    return;
                }

                var oldValue = _isLoading;
                _isLoading = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(IsLoadingPropertyName);
            }
        }
        #endregion

        #region IsMessagePostSuccess (bool)
        /// <summary>
        /// The <see cref="IsMessagePostSuccess" /> property's name.
        /// </summary>
        public const string IsMessagePostSuccessPropertyName = "IsMessagePostSuccess";

        private bool _isMessagePostSuccess = false;

        /// <summary>
        /// Gets the IsMessagePostSuccess property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsMessagePostSuccess
        {
            get
            {
                return _isMessagePostSuccess;
            }

            set
            {
                if (_isMessagePostSuccess == value)
                {
                    return;
                }

                var oldValue = _isMessagePostSuccess;
                _isMessagePostSuccess = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(IsMessagePostSuccessPropertyName);
            }
        }
        #endregion

        #region FBImagePreview (BitmapImage)
        /// <summary>
        /// The <see cref="FBImagePreview" /> property's name.
        /// </summary>
        public const string FBImagePreviewPropertyName = "FBImagePreview";

        private BitmapImage _imagePreview = null;

        /// <summary>
        /// Gets the FBImagePreview property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public BitmapImage FBImagePreview
        {
            get
            {
                return _imagePreview;
            }

            set
            {
                if (_imagePreview == value)
                {
                    return;
                }

                var oldValue = _imagePreview;
                _imagePreview = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(FBImagePreviewPropertyName);
            }
        }
        #endregion

        #region FBImageMessage (string)
        /// <summary>
        /// The <see cref="FBImageMessage" /> property's name.
        /// </summary>
        public const string FBImageMessagePropertyName = "FBImageMessage";

        private string _fbImageMessage = "If this works, it was officially too easy (working on a Windows Phone 7 Facebook tutorial)";

        /// <summary>
        /// Gets the FBImageMessage property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string FBImageMessage
        {
            get
            {
                return _fbImageMessage;
            }

            set
            {
                if (_fbImageMessage == value)
                {
                    return;
                }

                var oldValue = _fbImageMessage;
                _fbImageMessage = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(FBImageMessagePropertyName);
            }
        }
        #endregion

        #region FBImageSource (string)
        /// <summary>
        /// The <see cref="FBImageSource" /> property's name.
        /// </summary>
        public const string FBImageSourcePropertyName = "FBImageSource";

        private string _fbImageSource = "";

        /// <summary>
        /// Gets the FBImageSource property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string FBImageSource
        {
            get
            {
                return _fbImageSource;
            }

            set
            {
                if (_fbImageSource == value)
                {
                    return;
                }

                var oldValue = _fbImageSource;
                _fbImageSource = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(FBImageSourcePropertyName);
            }
        }
        #endregion

        #region HasImage (bool)
        /// <summary>
        /// The <see cref="HasImage" /> property's name.
        /// </summary>
        public const string HasImagePropertyName = "HasImage";

        private bool _hasImage = false;

        /// <summary>
        /// Gets the HasImage property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool HasImage
        {
            get
            {
                return _hasImage;
            }

            set
            {
                if (_hasImage == value)
                {
                    return;
                }

                var oldValue = _hasImage;
                _hasImage = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(HasImagePropertyName);
            }
        }
        #endregion

        #region SaveAccessToken Command (string accessToken)
        public ICommand SaveAccessToken
        {
            get
            {
                return new RelayCommand<string>(accessToken =>
                {
                    IsolatedStorageSettings Settings = IsolatedStorageSettings.ApplicationSettings;
                    if (Settings.Contains("MyFacebookAccessToken"))
                        Settings["MyFacebookAccessToken"] = accessToken;
                    else
                        Settings.Add("MyFacebookAccessToken", accessToken);
                    Settings.Save();

                    _asyncFbClient = new FacebookClient(accessToken);
                    _asyncFbClient.GetCompleted += new System.EventHandler<FacebookApiEventArgs>(_asyncFbClient_GetCompleted);
                    _asyncFbClient.GetAsync("/me");
                });
            }
        }

        void _asyncFbClient_GetCompleted(object sender, FacebookApiEventArgs e)
        {
            FacebookUser _fbUser = e.GetResultData<FacebookUser>();
            Deployment.Current.Dispatcher.BeginInvoke(()=>
            {
                FBUser = _fbUser;
                IsLoading = false;
            });
            _asyncFbClient.GetCompleted -= _asyncFbClient_GetCompleted;
        }
        #endregion

        #region SendFacebookMessage Command
        public ICommand SendFacebookMessage
        {
            get { return new RelayCommand(() => {
                    var parameters = new Dictionary<string, object>
                    {
                        {"message", FacebookMessage},
                        {"privacy", new Dictionary<string, object>
                            {
                                {"value", "ALL_FRIENDS"}
                            }
                        }
                    };
                    _asyncFbClient.PostCompleted += new System.EventHandler<FacebookApiEventArgs>(_asyncFbClient_PostCompleted);
                    _asyncFbClient.PostAsync("me/feed", parameters);
                    Deployment.Current.Dispatcher.BeginInvoke(() =>
                    {
                        IsLoading = true;
                    });
                });
            }
        }

        void _asyncFbClient_PostCompleted(object sender, FacebookApiEventArgs e)
        {
            if (!e.Cancelled && e.Error == null)
            {
                _asyncFbClient.PostCompleted -= _asyncFbClient_PostCompleted;
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    IsLoading = false;
                    IsMessagePostSuccess = true;
                    FacebookMessage = "";
                });
            }
            else
            {
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    IsLoading = false;
                    IsMessagePostSuccess = false;
                });
            }
        }
        #endregion

        #region ProcessImageResult Command calling ImageCrunching(PhotoResult result)
        public ICommand ProcessImageResult
        {
            get
            {
                return new RelayCommand<PhotoResult>(e =>
                {
                    ImageCrunching(e);
                });
            }
        }

        private object ImageCrunching(PhotoResult result)
        {
            _rawPhotoResult = result;
            Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                FBImagePreview = new BitmapImage();
                FBImagePreview.SetSource(ImageHelpers.FacebookImagePrep.RotateStream(result.ChosenPhoto, result.OriginalFileName, _additionalRotation));
                HasImage = true;
            });
            _fbImageHolder = ImageHelpers.FacebookImagePrep.RotatePhotoResult(result.ChosenPhoto, result.OriginalFileName, 720, _additionalRotation);

            return null;
        }
        #endregion

        #region RotateCounterClockWise calling MyCommandMethod
        public ICommand RotateCounterClockWise
        {
            get
            {
                return new RelayCommand(() =>
                {
                    _additionalRotation = _additionalRotation + 90;
                    if (_additionalRotation == 360)
                        _additionalRotation = 0;
                    Deployment.Current.Dispatcher.BeginInvoke(() =>
                    {
                        FBImagePreview = new BitmapImage();
                        FBImagePreview.SetSource(ImageHelpers.FacebookImagePrep.RotateStream(_rawPhotoResult.ChosenPhoto, _rawPhotoResult.OriginalFileName, _additionalRotation));
                    }); 
                    _fbImageHolder = ImageHelpers.FacebookImagePrep.RotatePhotoResult(_rawPhotoResult.ChosenPhoto, _rawPhotoResult.OriginalFileName, 720, _additionalRotation);

                });
            }
        }
        #endregion

        #region CommitFacebookPhoto Command
        public ICommand CommitFacebookPhoto
        {
            get
            {
                return new RelayCommand(() =>
                {
                    Deployment.Current.Dispatcher.BeginInvoke(() =>
                    {
                        IsLoading = true;
                    });
                    var facebookUpload = new Facebook.FacebookMediaObject
                    {
                        FileName = Guid.NewGuid() + ".jpg",
                        ContentType = "image/png"
                    };
                    facebookUpload.SetValue(_fbImageHolder);
                    var photoDetails = new Dictionary<string, object>();
                    photoDetails.Add("message", FBImageMessage);
                    photoDetails.Add("image", facebookUpload);

                    _asyncFbClient.PostCompleted += new EventHandler<FacebookApiEventArgs>(_asyncFbClient_ImagePostCompleted);
                    _asyncFbClient.PostAsync(@"/photos", photoDetails);

                });
            }
        }

        void _asyncFbClient_ImagePostCompleted(object sender, FacebookApiEventArgs e)
        {
            _asyncFbClient.PostCompleted -= _asyncFbClient_ImagePostCompleted;
            if (!e.Cancelled)
            {
                if (e.Error != null)
                {
                    Deployment.Current.Dispatcher.BeginInvoke(() =>
                    {
                        MessageBox.Show("Crap. Something went wrong and it was: " + e.Error.Message);
                    });
                }
                else
                {
                    _fbImageHolder = null;
                    Deployment.Current.Dispatcher.BeginInvoke(() =>
                    {
                        FBImagePreview = null;
                        FBImageMessage = "";
                        HasImage = false;
                        IsLoading = false;
                        MessageBox.Show("Image posted. You are awesome.");
                    });
                }


            }
            else
            {
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    MessageBox.Show("Hmmm. Looking like you screwed this one up. Gotta let the upload finish first, kiddo.");
                });
            }
        }
        #endregion
    }
}